@@ -55,16 +55,18 @@ namespace Orthanc
   {
     namespace fs = boost::filesystem;
 
-    output.SendOkHeader("text/html", false, 0, NULL);
-    output.SendString("<html>");
-    output.SendString("  <body>");
-    output.SendString("    <h1>Subdirectories</h1>");
-    output.SendString("    <ul>");
+    output.SetContentType("text/html");
+
+    std::string s;
+    s += "<html>";
+    s += "  <body>";
+    s += "    <h1>Subdirectories</h1>";
+    s += "    <ul>";
 
     if (uri.size() > 0)
     {
       std::string h = Toolbox::FlattenUri(uri) + "/..";
-      output.SendString("<li><a href=\"" + h + "\">..</a></li>");
+      s += "<li><a href=\"" + h + "\">..</a></li>";
     }
 
     fs::directory_iterator end;
@@ -78,12 +80,12 @@ namespace Orthanc
 
       std::string h = Toolbox::FlattenUri(uri) + "/" + f;
       if (fs::is_directory(it->status()))
-        output.SendString("<li><a href=\"" + h + "\">" + f + "</a></li>");
+        s += "<li><a href=\"" + h + "\">" + f + "</a></li>";
     }      
 
-    output.SendString("    </ul>");      
-    output.SendString("    <h1>Files</h1>");
-    output.SendString("    <ul>");
+    s += "    </ul>";      
+    s += "    <h1>Files</h1>";
+    s += "    <ul>";
 
     for (fs::directory_iterator it(p) ; it != end; ++it)
     {
@@ -95,12 +97,14 @@ namespace Orthanc
 
       std::string h = Toolbox::FlattenUri(uri) + "/" + f;
       if (fs::is_regular_file(it->status()))
-        output.SendString("<li><a href=\"" + h + "\">" + f + "</a></li>");
+        s += "<li><a href=\"" + h + "\">" + f + "</a></li>";
     }      
 
-    output.SendString("    </ul>");
-    output.SendString("  </body>");
-    output.SendString("</html>");
+    s += "    </ul>";
+    s += "  </body>";
+    s += "</html>";
+
+    output.SendBody(s);
   }
 
 
@@ -120,13 +124,7 @@ namespace Orthanc
   }
 
 
-  bool FilesystemHttpHandler::IsServedUri(const UriComponents& uri)
-  {
-    return Toolbox::IsChildUri(pimpl_->baseUri_, uri);
-  }
-
-
-  void FilesystemHttpHandler::Handle(
+  bool FilesystemHttpHandler::Handle(
     HttpOutput& output,
     HttpMethod method,
     const UriComponents& uri,
@@ -134,10 +132,16 @@ namespace Orthanc
     const Arguments& arguments,
     const std::string&)
   {
+    if (!Toolbox::IsChildUri(pimpl_->baseUri_, uri))
+    {
+      // This URI is not served by this handler
+      return false;
+    }
+
     if (method != HttpMethod_Get)
     {
-      output.SendMethodNotAllowedError("GET");
-      return;
+      output.SendMethodNotAllowed("GET");
+      return true;
     }
 
     namespace fs = boost::filesystem;
@@ -162,7 +166,9 @@ namespace Orthanc
     }
     else
     {
-      output.SendHeader(HttpStatus_404_NotFound);
+      output.SendStatus(HttpStatus_404_NotFound);
     }
+
+    return true;
   } 
 }
