@@ -38,6 +38,7 @@
 #include "../OrthancServer/ServerIndex.h"
 #include "../Core/Uuid.h"
 #include "../Core/DicomFormat/DicomNullValue.h"
+#include "../Core/FileStorage/FilesystemStorage.h"
 
 #include <ctype.h>
 #include <glog/logging.h>
@@ -220,11 +221,23 @@ TEST_P(DatabaseWrapperTest, Simple)
   index_->SetMetadata(a[4], MetadataType_ModifiedFrom, "TUTU");
   index_->ListAvailableMetadata(md, a[4]);
   ASSERT_EQ(2u, md.size());
+
+  std::map<MetadataType, std::string> md2;
+  index_->GetAllMetadata(md2, a[4]);
+  ASSERT_EQ(2u, md2.size());
+  ASSERT_EQ("TUTU", md2[MetadataType_ModifiedFrom]);
+  ASSERT_EQ("PINNACLE", md2[MetadataType_Instance_RemoteAet]);
+
   index_->DeleteMetadata(a[4], MetadataType_ModifiedFrom);
   index_->ListAvailableMetadata(md, a[4]);
   ASSERT_EQ(1u, md.size());
   ASSERT_EQ(MetadataType_Instance_RemoteAet, md.front());
 
+  index_->GetAllMetadata(md2, a[4]);
+  ASSERT_EQ(1u, md2.size());
+  ASSERT_EQ("PINNACLE", md2[MetadataType_Instance_RemoteAet]);
+
+
   ASSERT_EQ(21u + 42u + 44u, index_->GetTotalCompressedSize());
   ASSERT_EQ(42u + 42u + 44u, index_->GetTotalUncompressedSize());
 
@@ -557,8 +570,11 @@ TEST_P(DatabaseWrapperTest, LookupTagValue)
 TEST(ServerIndex, AttachmentRecycling)
 {
   const std::string path = "UnitTestsStorage";
+
   Toolbox::RemoveFile(path + "/index");
-  ServerContext context(path, ":memory:");   // The SQLite DB is in memory
+  FilesystemStorage storage(path);
+  ServerContext context(":memory:");   // The SQLite DB is in memory
+  context.SetStorageArea(storage);
   ServerIndex& index = context.GetIndex();
 
   index.SetMaximumStorageSize(10);
@@ -579,7 +595,13 @@ TEST(ServerIndex, AttachmentRecycling)
     instance.SetValue(DICOM_TAG_STUDY_INSTANCE_UID, "study-" + id);
     instance.SetValue(DICOM_TAG_SERIES_INSTANCE_UID, "series-" + id);
     instance.SetValue(DICOM_TAG_SOP_INSTANCE_UID, "instance-" + id);
-    ASSERT_EQ(StoreStatus_Success, index.Store(instance, attachments, ""));
+
+    std::map<MetadataType, std::string> instanceMetadata;
+    ServerIndex::MetadataMap metadata;
+    ASSERT_EQ(StoreStatus_Success, index.Store(instanceMetadata, instance, attachments, "", metadata));
+    ASSERT_EQ(2, instanceMetadata.size());
+    ASSERT_TRUE(instanceMetadata.find(MetadataType_Instance_RemoteAet) != instanceMetadata.end());
+    ASSERT_TRUE(instanceMetadata.find(MetadataType_Instance_ReceptionDate) != instanceMetadata.end());
 
     DicomInstanceHasher hasher(instance);
     ids.push_back(hasher.HashPatient());
